#include "PDLL.h"
#include "../dll_shared.h"
#include "dll_common.h"
#include "Vectors.h"

typedef struct fan_data_s {
	BITMAP*					pBmp;
	float					fAngle;
	float					fRate;
} fan_data_t, *fan_data_p;

entity_p SpawnReflection(string sType, string sName, int dwvPos[], string svVars[])
{
	int dwType = RT_UP;
	if(svVars[0] == "Up")
		dwType = RT_UP;
	if(svVars[0] == "Down")
		dwType = RT_DOWN;
	if(svVars[0] == "Left")
		dwType = RT_LEFT;
	if(svVars[0] == "Right")
		dwType = RT_RIGHT;
	if(svVars[0] == "Water")
		dwType = RT_WATER;

	g_pImports->m_CreateReflection(dwType, dwvPos[0], dwvPos[1], dwvPos[2], dwvPos[3]);

	return NULL;
}

void DrawFan(entity_p pEnt, BITMAP* pDest, int dwSX, int dwSY)
{
	fan_data_p pData = (fan_data_p)pEnt->pvData[0];
	g_pImports->m_RotateSprite(pDest, pData->pBmp, pEnt->fvPos[0]-dwSX-7, pEnt->fvPos[1]-dwSY-7, pData->fAngle);
}

void RunFan(entity_p pEnt, int dwTime)
{
	fan_data_p pFan = (fan_data_p)pEnt->pvData[0];

	pFan->fAngle += pFan->fRate;
	if(pFan->fAngle < 0.0f)
		pFan->fAngle += 256.0f;
	if(pFan->fAngle >= 256.0f)
		pFan->fAngle -= 256.0f;
}

entity_p SpawnFan(string sType, string sName, int dwvPos[], string svVars[])
{
	entity_p pEnt = g_pImports->m_NewEntity();

	pEnt->sName = sName;
	pEnt->sClass = sType;
	pEnt->fvPos[0] = (dwvPos[0]+dwvPos[2])>>1;
	pEnt->fvPos[1] = (dwvPos[1]+dwvPos[3])>>1;
	pEnt->fvVel[0] = 0.0f;
	pEnt->fvVel[1] = 0.0f;
	pEnt->fvSize[0] = 1.0f;
	pEnt->fvSize[1] = 1.0f;

	fan_data_p pData = new fan_data_t;
	pEnt->pvData[0] = pData;
	pData->pBmp = g_pImports->m_LoadSprite("Fan.bmp");
	pData->fAngle = 0.0f;
	pData->fRate = atof(svVars[0].c_str())*2.56f;

	pEnt->m_Run = RunFan;
	pEnt->m_Draw = DrawFan;

	g_pImports->m_EntityActive(pEnt);
	g_pImports->m_EntityVisible(pEnt);

	return pEnt;
}

void DrawAcidFlask(entity_p pEnt, BITMAP* pDest, int dwSX, int dwSY)
{
	g_pImports->m_DrawSprite(pDest, pEnt->pSprite, pEnt->fvPos[0]-dwSX-12, pEnt->fvPos[1]-dwSY-15);
}

void cbAcidSmokeRun(particle_s* pPtc, int dwTime)
{
	pPtc->fvVel[0] += ((rand()%11)-5)/30.0f;
}

bool cbAcidTouch(particle_s* pPtc, struct entity_s* pEnt, float* fvClip)
{
	if(rand()%10)
		return true;

	int dwR;
	particle_p pNP= g_pImports->m_CreateParticle();
	if(pNP)
	{
		pNP->bDraw = true;
		pNP->bGravity = false;
		pNP->bLarge = false;
		dwR = (rand()%128)+128;
		pNP->dwColor = makecol(dwR, dwR, dwR);
		pNP->fvVel[0] = 0.0f;
		pNP->fvVel[1] = -10.0f;
		pNP->fvPos[0] = pPtc->fvPos[0];
		pNP->fvPos[1] = pPtc->fvPos[1];
		pNP->dwDeath = g_pImports->dwGameTime+400+rand()%100;
		pNP->m_Run = cbAcidSmokeRun;
	}

	return true;
}

bool AcidFlaskTakeDamage(entity_p pEnt, particle_p pPtc, int dwDamage)
{
	pEnt->bGarbage = true;

	int dwR;
	particle_p pNP;
	for(int lp=0; lp<50; lp++)
	{
		pNP= g_pImports->m_CreateParticle();
		if(pNP)
		{
			pNP->bDraw = true;
			pNP->bGravity = true;
			pNP->bLarge = true;
			dwR = (rand()%128)+128;
			pNP->dwColor = makecol(0, dwR, 0);
			pNP->fvVel[0] = (rand()%201)-100;
			pNP->fvVel[1] = (rand()%201)-150;
			pNP->fvPos[0] = pEnt->fvPos[0]+(rand()%24)-12;
			pNP->fvPos[1] = pEnt->fvPos[1]+(rand()%12);
			pNP->dwDeath = g_pImports->dwGameTime+300+rand()%100;
			pNP->m_Touch = cbAcidTouch;
		}
	}

	return true;
}

entity_p SpawnAcidFlask(string sType, string sName, int dwvPos[], string svVars[])
{
	entity_p pEnt = g_pImports->m_NewEntity();

	pEnt->sName = sName;
	pEnt->sClass = sType;
	pEnt->fvPos[0] = (dwvPos[0]+dwvPos[2])>>1;
	pEnt->fvPos[1] = (dwvPos[1]+dwvPos[3])>>1;
	pEnt->fvVel[0] = 0.0f;
	pEnt->fvVel[1] = 0.0f;
	pEnt->fvSize[0] = 12.0f;
	pEnt->fvSize[1] = 16.0f;
	pEnt->bSolid = true;
	pEnt->pSprite = g_pImports->m_LoadSprite("AcidFlask.bmp");

	pEnt->m_Draw = DrawAcidFlask;
	pEnt->m_TakeDamage = AcidFlaskTakeDamage;

	g_pImports->m_EntityVisible(pEnt);
	g_pImports->m_EntitySolid(pEnt);
	g_pImports->m_EntityMobile(pEnt);

	return pEnt;
}
