#include "math.h"

#define ABS(a) ((a)<0?-(a):(a))

void Vec2Copy(const float* fvVec, float* fvOut)
{
	fvOut[0]=fvVec[0];
	fvOut[1]=fvVec[1];
}

void Vec2Add(const float* fvVec1, const float* fvVec2, float* fvOut)
{
	fvOut[0]=fvVec1[0]+fvVec2[0];
	fvOut[1]=fvVec1[1]+fvVec2[1];
}

void Vec2AddMul(const float* fvVec1, const float* fvVec2, float fVal, float* fvOut)
{
	fvOut[0]=fvVec1[0]+fvVec2[0]*fVal;
	fvOut[1]=fvVec1[1]+fvVec2[1]*fVal;
}

void Vec2Subtract(const float* fvVec1, const float* fvVec2, float* fvOut)
{
	fvOut[0]=fvVec1[0]-fvVec2[0];
	fvOut[1]=fvVec1[1]-fvVec2[1];
}

void Vec2Multiply(const float* fvVec1, const float* fvVec2, float* fvOut)
{
	fvOut[0]=fvVec1[0]*fvVec2[0];
	fvOut[1]=fvVec1[1]*fvVec2[1];
}

void Vec2Multiply(const float* fvVec1, const float fVal, float* fvOut)
{
	fvOut[0]=fvVec1[0]*fVal;
	fvOut[1]=fvVec1[1]*fVal;
}

void Vec2Divide(const float* fvVec1, const float* fvVec2, float* fvOut)
{
	fvOut[0]=fvVec1[0]/fvVec2[0];
	fvOut[1]=fvVec1[1]/fvVec2[1];
}

void Vec2Divide(const float* fvVec1, const float fVal, float* fvOut)
{
	fvOut[0]=fvVec1[0]/fVal;
	fvOut[1]=fvVec1[1]/fVal;
}

float Vec2Dot(const float* fvVec1, const float* fvVec2)
{
	return fvVec1[0]*fvVec2[0]+fvVec1[1]*fvVec2[1];
}

void Vec2Perp(const float* fvVec, float* fvOut)
{
	float x=-fvVec[1];
	float y=fvVec[0];
	fvOut[0]=x;
	fvOut[1]=y;
}

float Vec2IntersectDist(const float* fvLineOrigin, const float* fvLineNormal, const float* fvRayOrigin, const float* fvRayVec)
{
	double d = -Vec2Dot(fvLineNormal, fvLineOrigin);
	double numer = Vec2Dot(fvLineNormal, fvRayOrigin) + d;
	double denom = Vec2Dot(fvLineNormal, fvRayVec);
	return (float)-(numer / denom);
}

float Vec2IntersectDist(const float fLineOrigin, const float* fvLineNormal, const float* fvRayOrigin, const float* fvRayVec)
{
	double numer = Vec2Dot(fvLineNormal, fvRayOrigin) + fLineOrigin;
	double denom = Vec2Dot(fvLineNormal, fvRayVec);
	return (float)-(numer / denom);
}

float Vec2IntersectPoint(const float* fvLineOrigin, const float* fvLineNormal, const float* fvRayOrigin, const float* fvRayVec, float* fvOut)
{
	float fvTemp[2];
	float fDist=Vec2IntersectDist(fvLineOrigin, fvLineNormal, fvRayOrigin, fvRayVec);
	Vec2Multiply(fvRayVec, fDist, fvTemp);
	Vec2Add(fvRayOrigin, fvTemp, fvOut);
	return fDist;
}

float Vec2IntersectPoint(const float fLineOrigin, const float* fvLineNormal, const float* fvRayOrigin, const float* fvRayVec, float* fvOut)
{
	float fvTemp[2];
	float fDist=Vec2IntersectDist(fLineOrigin, fvLineNormal, fvRayOrigin, fvRayVec);
	Vec2Multiply(fvRayVec, fDist, fvTemp);
	Vec2Add(fvRayOrigin, fvTemp, fvOut);
	return fDist;
}

float Vec2LineDist(const float* fvLineOrigin, const float* fvLineNormal, const float* fvPoint)
{
	float d = -Vec2Dot(fvLineNormal, fvLineOrigin);
	return Vec2Dot(fvLineNormal, fvPoint) + d;
}

float Vec2LineDist(const float fLineOrigin, const float* fvLineNormal, const float* fvPoint)
{
	return Vec2Dot(fvLineNormal, fvPoint) + fLineOrigin;
}

void Vec2ClosestPoint(const float fLineOrigin, const float* fvLineNormal, const float* fvPoint, float *fvOut)
{
	float fDist = Vec2LineDist(fLineOrigin, fvLineNormal, fvPoint);
	Vec2Multiply(fvLineNormal, fDist, fvOut);
	Vec2Subtract(fvPoint, fvOut, fvOut);
}

float Vec2Length(const float* fvVec)
{
	return (float)sqrt(fvVec[0]*fvVec[0]+fvVec[1]*fvVec[1]);
}

float Vec2LengthEx(const float* fvVec)
{
	return fvVec[0]*fvVec[0]+fvVec[1]*fvVec[1];
}

void Vec2Normalize(const float* fvVec, float* fvOut)
{
	Vec2Divide(fvVec, Vec2Length(fvVec), fvOut);
}

void Vec2SetLength(const float* fvVec, float fLength, float* fvOut)
{
	Vec2Multiply(fvVec, fLength/Vec2Length(fvVec), fvOut);
}

void Vec2Invert(const float* fvVec, float* fvOut)
{
	fvOut[0]=-fvVec[0];
	fvOut[1]=-fvVec[1];
}

float Vec2IntersectCircle(const float* fvCentre, float fRadius, const float* fvRayOrigin, const float* fvRayVec)
{
	float Q[2];
	Vec2Subtract(fvCentre, fvRayOrigin, Q);
	float c = Vec2LengthEx(Q);
	float v = Vec2Dot(Q, fvRayVec);
	float d = fRadius*fRadius - (c - v*v);

	if (d < 0.0f) return -1.0f;

	return v - (float)sqrt(d);
} 

float Vec2IntersectInvCircle(const float* fvCentre, float fRadius, const float* fvRayOrigin, const float* fvRayVec)
{
	float Q[2];
	Vec2Subtract(fvCentre, fvRayOrigin, Q);
	float c = Vec2LengthEx(Q);
	float v = Vec2Dot(Q, fvRayVec);
	float d = fRadius*fRadius - (c - v*v);

	if (d < 0.0f) return -1.0f;

	return v + (float)sqrt(d);
} 
