#include <allegro.h>
#include <winalleg.h>
#include "AppPlatform.h"
#include "Vectors.h"
#include "DLLFuncs.h"
#include "Particles.h"
#include "Reflect.h"
#include "bmpmanager.h"

typedef dll_exports_p (*dll_load_func_p)(dll_imports_p);

tileset_p g_pvTileSets[64];
int g_dwTileSetCount;
dll_imports_p g_pImports = NULL;
tilemap_p g_pMap = NULL;
collision_map_p g_pColMap = NULL;
collision_quadmap_p g_pQuadMap = NULL;
BITMAP* g_pBack = NULL;

CApp* cbQuitPlatform(CApp* pApp, bool bUp, int dwKey)
{
	return NULL;
}

CApp* cbKeyFuncCaller(CApp* pApp, bool bUp, int dwKey)
{
	if(g_pImports->pvKeyFuncs[dwKey|(bUp?0x80:0x00)])
		g_pImports->pvKeyFuncs[dwKey|(bUp?0x80:0x00)](bUp, dwKey);

	return pApp;
}

CAppPlatform::CAppPlatform()
{
	string sDir = "Data\\Maps\\Hub\\";

	int lp;
	for(lp=0; lp<256; lp++)
	{
		pvKeyFuncs[lp] = cbKeyFuncCaller;
	}

	pvKeyFuncs[KEY_ESC] = cbQuitPlatform;

	set_config_file((sDir+"Map.cfg").c_str());

	int dwArgc;
	char** szvArgv;
	szvArgv = get_config_argv("", "Tiles", &dwArgc);
	string sTiles;
	for(lp=0; lp<dwArgc; lp++)
	{
		sTiles = szvArgv[lp];
		if(sTiles[0] == '%')
		{
			int dwBCD = bitmap_color_depth(screen);
			set_color_depth(32);
			BITMAP* pBmp = load_tga(("Data\\Tiles\\"+sTiles.substr(1)+".tga").c_str(), NULL);
			set_color_depth(dwBCD);

			g_pvTileSets[lp] = BuildTiles(pBmp);
			g_pvTileSets[lp]->bTransparent = true;
		}
		else
		{
			BITMAP* pBmp = load_bitmap(("Data\\Tiles\\"+sTiles+".bmp").c_str(), NULL);
			g_pvTileSets[lp] = BuildTiles(pBmp);
			g_pvTileSets[lp]->bTransparent = false;
		}
	}
	g_dwTileSetCount = dwArgc;

	g_pMap = LoadTileMap(sDir);
	g_pColMap = LoadCollisionMap(sDir);
	g_pQuadMap = BuildCollisionQuadMap(g_pColMap, g_pMap->dwSize[0]<<4, g_pMap->dwSize[1]<<4, 64);

	bShowMouse = true;

	g_pImports = BuildDLLImport();
	g_pImports->sMap = "Hub";
	
	al_ffblk sFB;
	if(!al_findfirst("Data\\DLL\\*.dll", &sFB, FA_RDONLY|FA_ARCH|FA_SYSTEM))
	{
		do {
			LoadDLL((string)"Data\\DLL\\"+sFB.name);
		} while(!al_findnext(&sFB));
		al_findclose(&sFB);
	}

	CreateEntityState();
	LoadEntities(g_pImports->pvSpawnFuncs, sDir);

	BITMAP** pBmp = LoadBmp("Data\\Back\\"+(string)get_config_string(NULL, "back", ""));
	if(pBmp)
		g_pBack = *pBmp;
	else
		g_pBack = NULL;
}

CAppPlatform::~CAppPlatform()
{
	delete g_pImports;
	for(dll_info_list_t::iterator pDLL=pvDLL.begin(); pDLL!=pvDLL.end();)
	{
		FreeLibrary((*pDLL)->hmDLL);
		delete (*pDLL)->pExports;
		
		pDLL = pvDLL.erase(pDLL);
	}
}

CApp* CAppPlatform::Run(int dwTime)
{
	g_pImports->dwGameTime = dwTime;
	g_pImports->dwvMousePos[0] = mouse_x;
	g_pImports->dwvMousePos[1] = mouse_y;
	g_pImports->bvMouseButtons[0] = (mouse_b&1)?true:false;
	g_pImports->bvMouseButtons[1] = (mouse_b&2)?true:false;
	g_pImports->bvMouseButtons[2] = (mouse_b&4)?true:false;
	RunEntities(dwTime);
	RunPlatforms(g_pQuadMap);
	RunParticles(dwTime);
	MoveEntities(g_pQuadMap);
	MoveParticles(g_pQuadMap, dwTime);
	RunTriggers(g_pImports->pPlayer);

	return this;
}

bool CAppPlatform::Draw(BITMAP* pBmp)
{
	static bool bvBack[4] = { true, true, false, false };
	static bool bvFront[4] = { false, false, true, true };

	if(g_pBack)
	{
		drawing_mode(DRAW_MODE_COPY_PATTERN, g_pBack, -g_pImports->dwvScrollPos[0]/2, -g_pImports->dwvScrollPos[1]/2);
		rectfill(pBmp, 0, 0, pBmp->w, pBmp->h, makecol(255, 255, 255));
		solid_mode();
	}
	else
	{
		clear_to_color(pBmp, makecol(128, 128, 255));
	}

//	GradientFill(pBmp);

	DrawTileMap(pBmp, g_pMap, g_pvTileSets, g_pImports->dwvScrollPos[0], g_pImports->dwvScrollPos[1], bvBack);
//	if(key[KEY_C])
//		DrawCollisionMap(pBmp, pColMap, -pImports->dwvScrollPos[0], -pImports->dwvScrollPos[1]);
	DrawEntities(pBmp, g_pImports->dwvScrollPos[0], g_pImports->dwvScrollPos[1]);
	DrawParticles(pBmp, g_pImports->dwvScrollPos[0], g_pImports->dwvScrollPos[1]);
	RenderReflections(pBmp, g_pImports->dwvScrollPos[0], g_pImports->dwvScrollPos[1]);
	DrawTileMap(pBmp, g_pMap, g_pvTileSets, g_pImports->dwvScrollPos[0], g_pImports->dwvScrollPos[1], bvFront);

	return true;
}

void CAppPlatform::LoadDLL(string sFile)
{
	dll_info_p pDLL = new dll_info_t;
	pDLL->sFile = sFile;

	pDLL->hmDLL = LoadLibrary(sFile.c_str());
	if(FAILED(pDLL->hmDLL))
	{
		delete pDLL;
		return;
	}

	dll_load_func_p pFunc = (dll_load_func_p)GetProcAddress(pDLL->hmDLL, "LoadGameDLL");
	if(!pFunc)
	{
		FreeLibrary(pDLL->hmDLL);
		delete pDLL;
		return;
	}

	pDLL->pExports = pFunc(g_pImports);
	if(!pDLL->pExports)
	{
		FreeLibrary(pDLL->hmDLL);
		delete pDLL;
		return;
	}

	pvDLL.push_back(pDLL);
}

void CAppPlatform::GradientFill(BITMAP* pBmp)
{
}
