#include <allegro.h>
#include <winalleg.h>
#include "Entity.h"
#include "Vectors.h"
#include <algorithm>

using namespace std;

extern collision_quadmap_p g_pQuadMap;

entity_state_p g_pEntState = NULL;
trigger_zone_list_t g_pvTriggers;

void CreateEntityState()
{
	g_pEntState = new entity_state_t;
}

entity_p NewEntity()
{
	entity_p pEnt = new entity_t;
	memset(pEnt, 0, sizeof(entity_t));

	pEnt->fvGravityMultiplier[0] = 1.0f;
	pEnt->fvGravityMultiplier[1] = 1.0f;

	g_pEntState->svEnts.push_back(pEnt);

	return pEnt;
}

void EntityVisible(entity_p pEnt)
{
	pEnt->bVisible = true;
	if(find(g_pEntState->svVisible.begin(), g_pEntState->svVisible.end(), pEnt) == g_pEntState->svVisible.end())
	{
		g_pEntState->svVisible.push_back(pEnt);
	}
}

void EntityInvisible(entity_p pEnt)
{
	pEnt->bVisible = false;
	g_pEntState->svVisible.erase(remove(g_pEntState->svVisible.begin(), g_pEntState->svVisible.end(), pEnt), g_pEntState->svVisible.end());
}

void EntityActive(entity_p pEnt)
{
	pEnt->bActive = true;
	if(find(g_pEntState->svActive.begin(), g_pEntState->svActive.end(), pEnt) == g_pEntState->svActive.end())
	{
		g_pEntState->svActive.push_back(pEnt);
	}
}

void EntityInactive(entity_p pEnt)
{
	pEnt->bActive = false;
	g_pEntState->svActive.erase(remove(g_pEntState->svActive.begin(), g_pEntState->svActive.end(), pEnt), g_pEntState->svActive.end());
}

void EntityAnimated(entity_p pEnt)
{
	pEnt->bAnimated = true;
	if(find(g_pEntState->svAnimated.begin(), g_pEntState->svAnimated.end(), pEnt) == g_pEntState->svAnimated.end())
	{
		g_pEntState->svAnimated.push_back(pEnt);
	}
}

void EntityInanimate(entity_p pEnt)
{
	pEnt->bAnimated = false;
	g_pEntState->svAnimated.erase(remove(g_pEntState->svAnimated.begin(), g_pEntState->svAnimated.end(), pEnt), g_pEntState->svAnimated.end());
}

void EntityMobile(entity_p pEnt)
{
	pEnt->bMobile = true;
	if(find(g_pEntState->svMobile.begin(), g_pEntState->svMobile.end(), pEnt) == g_pEntState->svMobile.end())
	{
		g_pEntState->svMobile.push_back(pEnt);
	}
}

void EntityStatic(entity_p pEnt)
{
	pEnt->bMobile = false;
	g_pEntState->svMobile.erase(remove(g_pEntState->svMobile.begin(), g_pEntState->svMobile.end(), pEnt), g_pEntState->svMobile.end());
}

void EntitySolid(entity_p pEnt)
{
	RemoveEntityFromNode(pEnt);
	AddEntityToNode(g_pQuadMap->pBaseNode, pEnt);

	pEnt->bCollider = true;
	if(find(g_pEntState->svSolid.begin(), g_pEntState->svSolid.end(), pEnt) == g_pEntState->svSolid.end())
	{
		g_pEntState->svSolid.push_back(pEnt);
	}
}

void EntityNonsolid(entity_p pEnt)
{
	RemoveEntityFromNode(pEnt);

	pEnt->bCollider = false;
	g_pEntState->svSolid.erase(remove(g_pEntState->svSolid.begin(), g_pEntState->svSolid.end(), pEnt), g_pEntState->svSolid.end());
}

void DrawEntities(BITMAP* pBmp, int dwX, int dwY)
{
	for(entity_list_t::iterator iPos=g_pEntState->svVisible.begin(); iPos!=g_pEntState->svVisible.end(); ++iPos)
	{
		if((*iPos)->m_Draw  &&  *iPos != g_pImports->pPlayer)
			(*iPos)->m_Draw(*iPos, pBmp, dwX, dwY);
//		ellipse(pBmp, (*iPos)->fvPos[0]-dwX, (*iPos)->fvPos[1]-dwY, (*iPos)->fvSize[0], (*iPos)->fvSize[1], makecol(255, rand()%256, 0));
	}
	if(g_pImports->pPlayer)
		g_pImports->pPlayer->m_Draw(g_pImports->pPlayer, pBmp, dwX, dwY);
}

typedef struct em_plane_s {
	float					fvPlane[4];
} em_plane_t, *em_plane_p;

void MoveEntity(entity_p pEnt, collision_quadmap_p pMap, float fvODir[], float* fvGround)
{
	fvGround[0] = fvGround[1] = fvGround[2] = fvGround[3] = 0.0f;
	float fvMove[2] = {
		pEnt->fvVel[0]*0.01f,
		pEnt->fvVel[1]*0.01f,
	};
	if(fvMove[0] == 0.0f  &&  fvMove[1] == 0.0f)
	{
		fvODir[0] = fvODir[1] = 0.0f;
		return;
	}
	Vec2Normalize(fvMove, fvODir);
	float fX = pEnt->fvPos[0]+fvMove[0];
	float fY = pEnt->fvPos[1]+fvMove[1];
	float fMove = 1.0f;
	float fTime = 0.0f;
	float fOX, fOY;
	float fvClip[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
	float fvTemp[2];
	int dwPlaneCount = 0;
	float fvDir[2];
	Vec2Normalize(fvMove, fvDir);
	while(1)
	{
		fMove = 1.0f;
		fOX = fX;
		fOY = fY;
		fvClip[2] = 0.0f;
		fvClip[3] = 0.0f;
		fMove = CheckLine_Blocker(pMap, pEnt->fvPos[0], pEnt->fvPos[1], fX, fY, pEnt->fvSize[0], pEnt->fvSize[1], fTime, fvClip);

		pEnt->fvPos[0] = fX;
		pEnt->fvPos[1] = fY;

		Vec2Add(fvGround, fvClip, fvGround);
		if(fvClip[2] != 0.0f  || fvClip[3] != 0.0f)
			Vec2Copy(fvClip+2, fvGround+2);

		if(fMove >= 0.999f)
		{
			return;
		}

		if(fMove > 0.0f)
		{
			fTime += (1.0f-fTime)*fMove;
			Vec2Multiply(fvMove, 1.0f-fMove, fvMove);
			dwPlaneCount = 0;
		}

		if(dwPlaneCount == 1)
		{
			fvODir[0] = fvODir[1] = 0.0f;
			return;
		}

		pEnt->fvPos[0] += 0.00125f*fvClip[0];
		pEnt->fvPos[1] += 0.00125f*fvClip[1];
		dwPlaneCount++;
 
		if(dwPlaneCount == 1)
		{
			Vec2Multiply(fvClip, Vec2Dot(fvClip, fvMove), fvTemp);
			Vec2Subtract(fvMove, fvTemp, fvMove);
		}

		if(Vec2Dot(fvMove, fvDir) <= 0.0f  ||  fvMove[0] == 0.0f  &&  fvMove[1] == 0.0f)
		{
			fvODir[0] = fvODir[1] = 0.0f;
			return;
		}

		Vec2Normalize(fvMove, fvODir);

		fX = fvMove[0]+pEnt->fvPos[0];
		fY = fvMove[1]+pEnt->fvPos[1];
	}
}

void MoveEntities(collision_quadmap_p pMap)
{
	float fvODir[2];
	float fVel;
	float fGL;
	float fvTemp[2];
	for(entity_list_t::iterator iPos=g_pEntState->svMobile.begin(); iPos!=g_pEntState->svMobile.end(); ++iPos)
	{
		(*iPos)->fvForce[1] += 250.0f*(*iPos)->fvGravityMultiplier[1];

		(*iPos)->fvVel[0] += (*iPos)->fvForce[0]*0.01f;
		(*iPos)->fvVel[1] += (*iPos)->fvForce[1]*0.01f;

		if((*iPos)->bSolid)
		{
			MoveEntity(*iPos, pMap, fvODir, (*iPos)->fvGround);
			fGL = Vec2Length((*iPos)->fvGround);
			if(fGL >= 0.01f)
			{
				Vec2Divide((*iPos)->fvGround, fGL, (*iPos)->fvGround);
				if((*iPos)->m_Touch)
					(*iPos)->m_Touch(*iPos, NULL);
			}
			fVel = Vec2Dot(fvODir, (*iPos)->fvVel);
			Vec2Multiply(fvODir, fVel, (*iPos)->fvVel);
			Vec2Multiply((*iPos)->fvGround, Vec2Dot((*iPos)->fvGround, (*iPos)->fvGround+2), fvTemp);
			Vec2Add((*iPos)->fvVel, fvTemp, (*iPos)->fvVel);
		}
		else
		{
			Vec2AddMul((*iPos)->fvPos, (*iPos)->fvVel, 0.01f, (*iPos)->fvPos);
		}

		if((*iPos)->bCollider)
		{
			RemoveEntityFromNode(*iPos);
			AddEntityToNode(g_pQuadMap->pBaseNode, *iPos);
		}

		(*iPos)->fvForce[0] = 0.0f;
		(*iPos)->fvForce[1] = 0.0f;
	}
}

void RunEntities(int dwTime)
{
	entity_list_t::iterator iPos;
	for(iPos=g_pEntState->svActive.begin(); iPos!=g_pEntState->svActive.end(); ++iPos)
	{
		if((*iPos)->m_Run)
			(*iPos)->m_Run(*iPos, dwTime);
	}
	for(iPos=g_pEntState->svEnts.begin(); iPos!=g_pEntState->svEnts.end();)
	{
		if((*iPos)->bGarbage)
		{
			EntityInactive(*iPos);
			EntityInanimate(*iPos);
			EntityInvisible(*iPos);
			EntityStatic(*iPos);
			EntityNonsolid(*iPos);
			iPos = g_pEntState->svEnts.erase(iPos);
		}
		else
		{
			++iPos;
		}
	}
}

void gb_fgets(string& sBuff, PACKFILE* pFile)
{
	char szBuff[256];
	int dwSize;
	pack_fread(&dwSize, sizeof(int), pFile);
	pack_fread(szBuff, dwSize, pFile);
	sBuff = szBuff;
}

void LoadEntities(spawn_func_list_t& pvSpawnFuncs, string sDir)
{
	PACKFILE* pFile = pack_fopen((sDir+"Ents.me").c_str(), "r");
	if(!pFile)
		return;

	int dwVersion;
	pack_fread(&dwVersion, sizeof(int), pFile);

	int dwCount;
	pack_fread(&dwCount, sizeof(int), pFile);

	string sType, sName;
	int dwvPos[4];
	string svVars[8];
	spawn_func_p pFunc;
	for(int lp=0; lp<dwCount; lp++)
	{
		gb_fgets(sType, pFile);
		gb_fgets(sName, pFile);
		pack_fread(&dwvPos[0], sizeof(int), pFile);
		pack_fread(&dwvPos[1], sizeof(int), pFile);
		pack_fread(&dwvPos[2], sizeof(int), pFile);
		pack_fread(&dwvPos[3], sizeof(int), pFile);
		gb_fgets(svVars[0], pFile);
		gb_fgets(svVars[1], pFile);
		gb_fgets(svVars[2], pFile);
		gb_fgets(svVars[3], pFile);
		gb_fgets(svVars[4], pFile);
		gb_fgets(svVars[5], pFile);
		gb_fgets(svVars[6], pFile);
		gb_fgets(svVars[7], pFile);

		if(pFunc = pvSpawnFuncs[sType.c_str()])
		{
			pFunc(sType, sName, dwvPos, svVars);
		}
	}

	pack_fclose(pFile);
}

void RunTriggers(entity_p pPlayer)
{
	if(!pPlayer)
		return;

	float fX1 = pPlayer->fvPos[0]-pPlayer->fvSize[0];
	float fY1 = pPlayer->fvPos[1]-pPlayer->fvSize[1];
	float fX2 = pPlayer->fvPos[0]+pPlayer->fvSize[0];
	float fY2 = pPlayer->fvPos[1]+pPlayer->fvSize[1];
	for(trigger_zone_list_t::iterator iPos=g_pvTriggers.begin(); iPos!=g_pvTriggers.end(); ++iPos)
	{
		if(iPos->bActive  &&  iPos->m_Trigger  &&  fX1 > iPos->fvPos[0]  &&  fY1 > iPos->fvPos[1]  &&  fX2 < iPos->fvPos[2]  &&  fY2 < iPos->fvPos[3])
		{
			iPos->m_Trigger(&(*iPos), pPlayer, iPos->pData);
		}
	}
}

trigger_zone_p AddTrigger()
{
	trigger_zone_t sZone;
	sZone.bActive = false;
	sZone.m_Trigger = NULL;
	sZone.pData = NULL;

	g_pvTriggers.push_front(sZone);

	return &g_pvTriggers.front();
}

entity_p FindEntityByName(string sName, entity_p pStart)
{
	entity_list_t::iterator pEnt;
	if(pStart)
		pEnt = ++find(g_pEntState->svEnts.begin(), g_pEntState->svEnts.end(), pStart);
	else
		pEnt = g_pEntState->svEnts.begin();

	for(; pEnt!=g_pEntState->svEnts.end(); ++pEnt)
	{
		if((*pEnt)->sName == sName)
			return *pEnt;
	}

	return NULL;
}

entity_p FindEntityAtPoint(float fX, float fY, entity_p pStart)
{
	entity_list_t::iterator pEnt;
	if(pStart)
		pEnt = ++find(g_pEntState->svEnts.begin(), g_pEntState->svEnts.end(), pStart);
	else
		pEnt = g_pEntState->svEnts.begin();

	for(; pEnt!=g_pEntState->svEnts.end(); ++pEnt)
	{
		if(fX >= (*pEnt)->fvPos[0]-(*pEnt)->fvSize[0]  &&
			fX <= (*pEnt)->fvPos[0]+(*pEnt)->fvSize[0]  &&
			fY >= (*pEnt)->fvPos[1]-(*pEnt)->fvSize[1]  &&
			fY <= (*pEnt)->fvPos[1]+(*pEnt)->fvSize[1])
		{
			return *pEnt;
		}
	}

	return NULL;
}

entity_p FindEntityInBox(float fX1, float fY1, float fX2, float fY2, entity_p pStart)
{
	entity_list_t::iterator pEnt;
	if(pStart)
		pEnt = ++find(g_pEntState->svEnts.begin(), g_pEntState->svEnts.end(), pStart);
	else
		pEnt = g_pEntState->svEnts.begin();

	for(; pEnt!=g_pEntState->svEnts.end(); ++pEnt)
	{
		if((*pEnt)->fvPos[0]-(*pEnt)->fvSize[0] <= fX2  &&
			(*pEnt)->fvPos[0]+(*pEnt)->fvSize[0] >= fX1  &&
			(*pEnt)->fvPos[1]-(*pEnt)->fvSize[1] <= fY2  &&
			(*pEnt)->fvPos[1]+(*pEnt)->fvSize[1] >= fY1)
		{
			return *pEnt;
		}
	}

	return NULL;
}

void ClearEntities(entity_p pPlayer)
{
	for(entity_list_t::iterator iPos=g_pEntState->svEnts.begin(); iPos!=g_pEntState->svEnts.end();)
	{
		if(*iPos != pPlayer)
		{
			EntityInactive(*iPos);
			EntityInanimate(*iPos);
			EntityInvisible(*iPos);
			EntityStatic(*iPos);
			iPos = g_pEntState->svEnts.erase(iPos);
		}
		else
		{
			++iPos;
		}
	}
}
