#include <allegro.h>
#include <winalleg.h>
#include "Tile.h"

tileset_p BuildTiles(BITMAP* pBmp)
{
	tileset_p pTiles = new tileset_t;

	pTiles->pMainBmp = pBmp;
	pTiles->dwWidth = pBmp->w>>4;
	pTiles->dwHeight = pBmp->h>>4;
	pTiles->dwCount = pTiles->dwWidth*pTiles->dwHeight;
	
	for(int lx=0; lx<pTiles->dwWidth; lx++)
		for(int ly=0; ly<pTiles->dwHeight; ly++)
			pTiles->pvTiles[lx+ly*pTiles->dwWidth] = create_sub_bitmap(pBmp, lx<<4, ly<<4, 16, 16);

	return pTiles;
}

void DeleteTiles(tileset_p pTiles)
{
	for(int lp=0; lp<pTiles->dwCount; lp++)
		destroy_bitmap(pTiles->pvTiles[lp]);

	destroy_bitmap(pTiles->pMainBmp);

	delete pTiles;
}

int GetPow(int dwVal)
{
	int dwRet = 0;
	while(dwVal > 1)
	{
		dwRet++;
		dwVal /= 2;
	}
	return dwRet;
}

tilemap_p LoadTileMap(string sDir)
{
	tilemap_p pMap = new tilemap_t;

	set_config_file((sDir+"Map.cfg").c_str());

	pMap->sName = get_config_string("", "Name", "Map");
	pMap->dwSize[0] = get_config_int("", "Width", 128);
	pMap->dwSize[1] = get_config_int("", "Height", 128);
	pMap->dwSize[2] = GetPow(pMap->dwSize[0]);
	pMap->dwSize[3] = GetPow(pMap->dwSize[1]);

	pMap->pvvLayers[0] = (short*)malloc(sizeof(short)*pMap->dwSize[0]*pMap->dwSize[1]);
	pMap->pvvLayers[1] = (short*)malloc(sizeof(short)*pMap->dwSize[0]*pMap->dwSize[1]);
	pMap->pvvLayers[2] = (short*)malloc(sizeof(short)*pMap->dwSize[0]*pMap->dwSize[1]);
	pMap->pvvLayers[3] = (short*)malloc(sizeof(short)*pMap->dwSize[0]*pMap->dwSize[1]);

	memset(pMap->pvvLayers[0], 0, sizeof(short)*pMap->dwSize[0]*pMap->dwSize[1]);
	memset(pMap->pvvLayers[1], 0, sizeof(short)*pMap->dwSize[0]*pMap->dwSize[1]);
	memset(pMap->pvvLayers[2], 0, sizeof(short)*pMap->dwSize[0]*pMap->dwSize[1]);
	memset(pMap->pvvLayers[3], 0, sizeof(short)*pMap->dwSize[0]*pMap->dwSize[1]);

	PACKFILE* pFile = pack_fopen((sDir+"TileMap.mpt").c_str(), "r");
	if(pFile)
	{
		int dwVersion;
		pack_fread(&dwVersion, sizeof(int), pFile);

		int dwvSize[2];
		pack_fread(&dwvSize[0], sizeof(int), pFile);
		pack_fread(&dwvSize[1], sizeof(int), pFile);

		int dwS = pMap->dwSize[0]*pMap->dwSize[1];
		pack_fread(pMap->pvvLayers[0], sizeof(short)*dwS, pFile);
		pack_fread(pMap->pvvLayers[1], sizeof(short)*dwS, pFile);
		pack_fread(pMap->pvvLayers[2], sizeof(short)*dwS, pFile);
		pack_fread(pMap->pvvLayers[3], sizeof(short)*dwS, pFile);

		pack_fclose(pFile);
	}

	return pMap;
}

void DrawTile(BITMAP* pBmp, tileset_p* pvTiles, int dwTile, int dwX, int dwY)
{
	tileset_p pPage = pvTiles[dwTile>>8];
	if(pPage->bTransparent)
		draw_trans_sprite(pBmp, pPage->pvTiles[dwTile&0xFF], dwX, dwY);
	else
		draw_sprite(pBmp, pPage->pvTiles[dwTile&0xFF], dwX, dwY);
}

void DrawTileMap(BITMAP* pBmp, tilemap_p pMap, tileset_p* pvTiles, int dwX, int dwY, bool* bvLayers)
{
	int dwEX = dwX>>4;
	int dwEY = dwY>>4;
	int dwOX = dwX-(dwEX<<4);
	int dwOY = dwY-(dwEY<<4);

	int dwW = (SCREEN_W>>4)+2;
	int dwH = (SCREEN_H>>4)+2;

	if(dwEX+dwW >= pMap->dwSize[0])
		dwW = pMap->dwSize[0]-dwEX-1;
	if(dwEY+dwH >= pMap->dwSize[1])
		dwH = pMap->dwSize[1]-dwEY-1;

	if(dwEX < 0)
	{
		dwOX += dwEX<<4;
		dwW += dwEX;
		dwEX = 0;
	}
	if(dwEY < 0)
	{
		dwOY += dwEY<<4;
		dwH += dwEY;
		dwEY = 0;
	}

	set_alpha_blender();

	int dwS = dwEX+dwEY*pMap->dwSize[0];
	short* iPos;
	int lx, ly;
	if(bvLayers[0])
	{
		for(ly=0; ly<dwH; ly++)
		{
			iPos = pMap->pvvLayers[0]+(dwS+(ly<<pMap->dwSize[2]));
			for(lx=0; lx<dwW; lx++)
			{
				if(*iPos)
					DrawTile(pBmp, pvTiles, *iPos, (lx<<4)-dwOX, (ly<<4)-dwOY);
				iPos++;
			}
		}
	}
	if(bvLayers[1])
	{
		for(ly=0; ly<dwH; ly++)
		{
			iPos = pMap->pvvLayers[1]+(dwS+(ly<<pMap->dwSize[2]));
			for(lx=0; lx<dwW; lx++)
			{
				if(*iPos)
					DrawTile(pBmp, pvTiles, *iPos, (lx<<4)-dwOX, (ly<<4)-dwOY);
				iPos++;
			}
		}
	}
	if(bvLayers[2])
	{
		for(ly=0; ly<dwH; ly++)
		{
			iPos = pMap->pvvLayers[2]+(dwS+(ly<<pMap->dwSize[2]));
			for(lx=0; lx<dwW; lx++)
			{
				if(*iPos)
					DrawTile(pBmp, pvTiles, *iPos, (lx<<4)-dwOX, (ly<<4)-dwOY);
				iPos++;
			}
		}
	}
	if(bvLayers[3])
	{
		for(ly=0; ly<dwH; ly++)
		{
			iPos = pMap->pvvLayers[3]+(dwS+(ly<<pMap->dwSize[2]));
			for(lx=0; lx<dwW; lx++)
			{
				if(*iPos)
					DrawTile(pBmp, pvTiles, *iPos, (lx<<4)-dwOX, (ly<<4)-dwOY);
				iPos++;
			}
		}
	}
}

void PlaceTile(tilemap_p pMap, tileset_p* pvTiles, int dwStart, int dwW, int dwH, int dwX, int dwY, int dwLayer)
{
	int dwPage = dwStart>>8;
	tileset_p pPage = pvTiles[dwPage];
	dwStart &= 0xFF;
	int dwTX = dwStart%pPage->dwWidth;
	int dwTY = dwStart/pPage->dwWidth;

	if(dwX < 0)
	{
		dwW += dwX;
		dwX = 0;
	}
	if(dwY < 0)
	{
		dwH += dwY;
		dwY = 0;
	}
	if(dwX+dwW >= pMap->dwSize[0])
		dwW = pMap->dwSize[0]-dwX-1;
	if(dwY+dwH >= pMap->dwSize[1])
		dwH = pMap->dwSize[1]-dwY-1;

	for(int lx=0; lx<dwW; lx++)
		for(int ly=0; ly<dwH; ly++)
			(pMap->pvvLayers[dwLayer])[(lx+dwX)+((ly+dwY)<<pMap->dwSize[2])] = (dwPage<<8)+(lx+dwTX)+(ly+dwTY)*pPage->dwWidth;
}

void DeleteTile(tilemap_p pMap, int dwW, int dwH, int dwX, int dwY, int dwLayer)
{
	if(dwX < 0)
	{
		dwW += dwX;
		dwX = 0;
	}
	if(dwY < 0)
	{
		dwH += dwY;
		dwY = 0;
	}
	if(dwX+dwW >= pMap->dwSize[0])
		dwW = pMap->dwSize[0]-dwX-1;
	if(dwY+dwH >= pMap->dwSize[1])
		dwH = pMap->dwSize[1]-dwY-1;

	for(int lx=0; lx<dwW; lx++)
		for(int ly=0; ly<dwH; ly++)
			(pMap->pvvLayers[dwLayer])[(lx+dwX)+((ly+dwY)<<pMap->dwSize[2])] = 0;
}

void SaveTileMap(string sDir, tilemap_p pMap)
{
	PACKFILE* pFile = pack_fopen((sDir+"TileMap.mpt").c_str(), "w");

	int dwVersion = 1;
	pack_fwrite(&dwVersion, sizeof(int), pFile);
	pack_fwrite(&pMap->dwSize[0], sizeof(int), pFile);
	pack_fwrite(&pMap->dwSize[1], sizeof(int), pFile);

	int dwS = pMap->dwSize[0]*pMap->dwSize[1];
	pack_fwrite(pMap->pvvLayers[0], sizeof(short)*dwS, pFile);
	pack_fwrite(pMap->pvvLayers[1], sizeof(short)*dwS, pFile);
	pack_fwrite(pMap->pvvLayers[2], sizeof(short)*dwS, pFile);
	pack_fwrite(pMap->pvvLayers[3], sizeof(short)*dwS, pFile);

	pack_fclose(pFile);
}

void DeleteTileMap(tilemap_p& pMap)
{
	delete [] pMap->pvvLayers[0];
	delete [] pMap->pvvLayers[1];
	delete [] pMap->pvvLayers[2];
	delete [] pMap->pvvLayers[3];

	delete pMap;

	pMap = NULL;
}
