#ifndef VECTORS_H
#define VECTORS_H

#define Vec2Normalise Vec2Normalize
#define Vec3Normalise Vec3Normalize

inline void Vec2Copy(const float* fvVec, float* fvOut);
inline void Vec2Add(const float* fvVec1, const float* fvVec2, float* fvOut);
inline void Vec2AddMul(const float* fvVec1, const float* fvVec2, float fVal, float* fvOut);
inline void Vec2Subtract(const float* fvVec1, const float* fvVec2, float* fvOut);
inline void Vec2Multiply(const float* fvVec1, const float* fvVec2, float* fvOut);
inline void Vec2Multiply(const float* fvVec1, const float fVal, float* fvOut);
inline void Vec2Divide(const float* fvVec1, const float* fvVec2, float* fvOut);
inline void Vec2Divide(const float* fvVec1, const float fVal, float* fvOut);
inline float Vec2Dot(const float* fvVec1, const float* fvVec2);
inline void Vec2Perp(const float* fvVec, float* fvOut);
float Vec2IntersectDist(const float* fvPlaneOrigin, const float* fvPlaneNormal, const float* fvRayOrigin, const float* fvRayVec);
float Vec2IntersectDist(const float fPlaneOrigin, const float* fvPlaneNormal, const float* fvRayOrigin, const float* fvRayVec);
float Vec2IntersectPoint(const float* fvPlaneOrigin, const float* fvPlaneNormal, const float* fvRayOrigin, const float* fvRayVec, float* fvOut);
float Vec2IntersectPoint(const float fPlaneOrigin, const float* fvPlaneNormal, const float* fvRayOrigin, const float* fvRayVec, float* fvOut);
float Vec2LineDist(const float* fvPlaneOrigin, const float* fvPlaneNormal, const float* fvPoint);
float Vec2LineDist(const float fPlaneOrigin, const float* fvPlaneNormal, const float* fvPoint);
void Vec2ClosestPoint(const float fLineOrigin, const float* fvLineNormal, const float* fvPoint, float *fvOut);
inline float Vec2Length(const float* fvVec);
inline float Vec2LengthEx(const float* fvVec);
inline void Vec2Normalize(const float* fvVec, float* fvOut);
inline void Vec2SetLength(const float* fvVec, float fLength, float* fvOut);
inline void Vec2Invert(const float* fvVec, float* fvOut);
float Vec2IntersectCircle(const float* fvCentre, float fRadius, const float* fvRayOrigin, const float* fvRayVec);
float Vec2IntersectInvCircle(const float* fvCentre, float fRadius, const float* fvRayOrigin, const float* fvRayVec);

inline void Vec3Copy(const float* fvVec, float* fvOut);
inline void Vec3Add(const float* fvVec1, const float* fvVec2, float* fvOut);
inline void Vec3Subtract(const float* fvVec1, const float* fvVec2, float* fvOut);
inline void Vec3Multiply(const float* fvVec1, const float* fvVec2, float* fvOut);
inline void Vec3Multiply(const float* fvVec1, const float fVal, float* fvOut);
inline void Vec3Divide(const float* fvVec1, const float* fvVec2, float* fvOut);
inline void Vec3Divide(const float* fvVec1, const float fVal, float* fvOut);
inline void Vec3Interpolate(const float* fvVec1, const float* fvVec2, const float fVal, float* fvOut);
inline float Vec3Dot(const float* fvVec1, const float* fvVec2);
inline void Vec3Cross(const float* fvVec1, const float* fvVec2, float* fvOut);
float Vec3IntersectDist(const float* fvLineOrigin, const float* fvLineNormal, const float* fvRayOrigin, const float* fvRayVec);
float Vec3IntersectPoint(const float* fvLineOrigin, const float* fvLineNormal, const float* fvRayOrigin, const float* fvRayVec, float* fvOut);
float Vec3IntersectDist(const float fLineOrigin, const float* fvLineNormal, const float* fvRayOrigin, const float* fvRayVec);
float Vec3IntersectPoint(const float fLineOrigin, const float* fvLineNormal, const float* fvRayOrigin, const float* fvRayVec, float* fvOut);
inline float Vec3LineDist(const float* fvLineOrigin, const float* fvLineNormal, const float* fvPoint);
inline float Vec3LineDist(const float fLineOrigin, const float* fvLineNormal, const float* fvPoint);
inline float Vec3Length(const float* fvVec);
inline float Vec3LengthEx(const float* fvVec);
inline void Vec3Normalize(const float* fvVec, float* fvOut);
inline void Vec3SetLength(const float* fvVec, float fLength, float* fvOut);
inline void Vec3Invert(const float* fvVec, float* fvOut);
float Vec3IntersectSphere(const float* fvCentre, float fRadius, const float* fvRayOrigin, const float* fvRayVec);
int Vec3ClipLine(const float fPlaneDist, const float* fvPlaneDir, const float* fvStart, const float* fvEnd, float* fvFS, float* fvFE, float* fvBS, float* fvBE);
int Vec3XClipLine(const float fPlaneDist, const float* fvStart, const float* fvEnd, float* fvFS, float* fvFE, float* fvBS, float* fvBE);
int Vec3YClipLine(const float fPlaneDist, const float* fvStart, const float* fvEnd, float* fvFS, float* fvFE, float* fvBS, float* fvBE);
int Vec3ZClipLine(const float fPlaneDist, const float* fvStart, const float* fvEnd, float* fvFS, float* fvFE, float* fvBS, float* fvBE);
void Vec3PlaneFromPoints(const float* fvP1, const float* fvP2, const float* fvP3, float& fPlaneDist, float* fvPlaneDir);

#define V3CL_FRONT					0
#define V3CL_BACK					1
#define V3CL_CROSS_FROM_FRONT		2
#define V3CL_CROSS_FROM_BACK		3

#endif
